#pragma once
//////////////////////////////////////////////////////////////////////////////////////
//																		            //
// SlopeCal.h - Calculate TI values                                                    //																		   
// ver 1.0                                                                          //
// Language:    Visual C++ 2010                                                     //
// Platform:                     										            //
// Application: iTree-Temperature, Winter 2013                                      //
// Programmer: Yang Yang, yyang3117@gmail.com				                        //																	   	
//////////////////////////////////////////////////////////////////////////////////////
/*
  

  Module Operations: 
  ==================
  constructor: WatershedData(Params &pa)  
  This class is to read the spatial information of a watershed
  DEM
  Land cover
  Impervious cover
  Tree canopy cover
  calculate the slope, aspect, accumulation area, TIe, TIn
  and assign these values to cells within the watershed

  Public Interface:
  =================
  WatershedData waterD(&pa);
  

  Build Process:
  ==============
  Required files
    - InputWatershed.cpp
	
  Build commands 
    - devenv PASATH.sln
   
  Maintenance History:
  ====================

 */



#include <string>
#include <iomanip>
#include <cstdlib>
#include <sys/stat.h>

#include <vector>
#include <math.h>
#include "TemperatureInputRepo.h"

class SlopeCal
{

  public:
	    
	SlopeCal(TemperatureInputs* input);
	void calSlopeAspect();
	double getSlope(int i){return Slope[i];}
	double getAspect(int i){return Aspect[i];}
	

   private:
	    TemperatureInputs *inp;
	    std::vector<std::vector<double>> DEM;
		int rows;
		int cols;
		double xllcorner;
		double yllcorner;
        double cellsize;    // cell edge length  
	    double NODATA;    // the nodata value in the DEM
        int NATB;        // total number of useful cells (the cells with elevation)
		std::vector<double> Slope;
		std::vector<double> Aspect;
};